#++
# FILE: REGISTER_CONFIG.PY -- 21-JAN-2000 - Uwe Zesisn
#
# registers additional modules in files CONFIG_MODULES.TXT, CONFIG_OLB.TXT and
#  CONFIG.DAT
# remember to re-run the following procedures after (de)registration:
#       $ @ PYTHON_TOOLS:CVT_CONFIG_MODULES.COM
#       $ @ PYTHON_TOOLS:CVT_CONFIG_OLB.COM
#	$ @ PYTHON_VMS:CONFIG_INITTAB2MAR.COM  "CONFIG.DAT"  "D"
#
# Parameters:
#       P1 = "R" - registers new component
#            "U" - un-registers component
#
#       P2 = filespec describing a new / old component
#
# Example:
#       $ python PYTHON_TOOLS:REGISTER_CONFIG.PY  R  -
#	$_	PYTHON_VMS:CONFIG_VMS_SMG.TXT
#
# -----
# 10-APR-2000 ZE. -- add support for CONFIG.DAT
#--

# -----------------------------------------------------------------------------
def regcfg__find_component_in_config \
		(t_p1_component, at_moddata, at_olbdata, at_cfgdata):
  #++
  # go over PYTHON_VMS:CONFIG_MODULES.TXT and CONFIG_OLB.TXT to see if
  # the component name specified in P1 has already been registered.
  #
  # Parameters:
  #     P1 = component name ("ZLIB")
  # Returns:
  #     REGCFG_GX_EXISTS == (-1, 0, 1) (error, not found, found)
  #--

  # --------------------
  # -- check if component is registered in PYTHON_VMS:CONFIG_MODULES.TXT

  l_mod_idx = 0
  while (l_mod_idx < len(at_moddata)):
    if (at_moddata[l_mod_idx] [0:12] == '!$COMPONENT='):
      t_component = at_moddata[l_mod_idx] [12:-1] # :-1 = remove trailing \n
      if (t_component == t_p1_component):
        x_component_modules = 1                 # still check CONFIG_OLB.TXT
        break
    l_mod_idx = l_mod_idx + 1
  else:
    x_component_modules = 0                     # still check CONFIG_OLB.TXT

  # --------------------
  # -- check if component is registered in PYTHON_VMS:CONFIG_OLB.TXT

  l_olb_idx = 0
  while (l_olb_idx < len(at_olbdata)):
    if (at_olbdata[l_olb_idx] [0:12] == '!$COMPONENT='):
      t_component = at_olbdata[l_olb_idx] [12:-1] # :-1 = remove trailing \n
      if (t_component == t_p1_component):
        x_component_olb = 1
        break
    l_olb_idx = l_olb_idx + 1
  else:
    x_component_olb = 0

  # --------------------
  # -- check if component is registered in PYTHON_VMS:CONFIG.DAT

  l_cfg_idx = 0
  while (l_cfg_idx < len(at_cfgdata)):
    if (at_cfgdata[l_cfg_idx] [0:12] == '!$COMPONENT='):
      t_component = at_cfgdata[l_cfg_idx] [12:-1] # :-1 = remove trailing \n
      if (t_component == t_p1_component):
        x_component_cfg = 1
        break
    l_cfg_idx = l_cfg_idx + 1
  else:
    x_component_cfg = 0

  # --------------------
  # -- component must exist in all 3 files
  # @@improve check
  if (x_component_modules):
    if (not x_component_olb):
      print '%REGISTER_CONFIG-F-INCONCFG, existing configuration is inconsistent'
      print ' component', t_p1_component, 'exists in "CONFIG_MODULES.TXT"'
      print ' but not "CONFIG_OLB.TXT"'
      return -1                         # error
    else:
      return 1                          # component occurs in both files
  else:
    if (x_component_olb):
      print '%REGISTER_CONFIG-F-INCONCFG, existing configuration is inconsistent'
      print ' component', t_p1_component, 'exists in "CONFIG_OLB.TXT"'
      print ' but not "CONFIG_MODULES.TXT"'
      return -1                         # error
    else:
      return 0                          # component does not occur in any file

# def regcfg__find_component_in_config (t_p1_component, at_moddata, at_olbdata, at_cfgdata):

# -----------------------------------------------------------------------------

def regcfg_reg_modules (t_p1_component, at_p2_regdata, at_moddata):
  #++
  # register modules from (string)-list |at_p2_regdata|
  #
  # Returns
  #   1 = success, data in PYTHON_VMS:CONFIG_MODULES.TMP
  #  -1 = error
  #--

  # --------------------
  # -- locate EOF marker
  l_mod_idx = 0
  CopyDone_Exceptn = 'CopyDone'
  try:
    while (l_mod_idx < len (at_moddata)):
      if (at_moddata[l_mod_idx] [0:6] == '!$EOF='):
        # insert here
        l_reg_idx = 0
        # find begin of modules data
        while (l_reg_idx < len (at_p2_regdata)):
          if (at_p2_regdata[l_reg_idx] [0:8] == '$MODULES'):
            # found, insert
            l_reg_idx = l_reg_idx + 1
            at_moddata.insert (l_mod_idx, '!$COMPONENT='+t_p1_component+'\n')
            l_mod_idx = l_mod_idx + 1
            while (l_reg_idx < len (at_p2_regdata)):
              if (at_p2_regdata[l_reg_idx] [0:11] == '$ENDMODULES'):
                at_moddata.insert (l_mod_idx, '!$ENDCOMPONENT='+t_p1_component+'\n')
                raise CopyDone_Exceptn  # signal end to outer loops
                break
              else:
                at_moddata.insert (l_mod_idx, at_p2_regdata[l_reg_idx])
                l_mod_idx = l_mod_idx + 1
                l_reg_idx = l_reg_idx + 1
            else:
              print '%regcfg_reg_modules-f-bugchk, missing $ENDMODULES marker'
              import sys; sys.exit(0x2C)
          l_reg_idx = l_reg_idx + 1
        else:
          print '%regcfg_reg_modules-f-bugchk, missing $MODULES marker'
          import sys; sys.exit(0x2C)
        #-while
        break
      l_mod_idx = l_mod_idx + 1
    else:
      print '%regcfg_reg_modules-f-bugchk, missing !$EOF marker'
      import sys; sys.exit(0x2C)
  except CopyDone_Exceptn:
    pass

  # write new data
  fil_cfgmodout = open ('python_vms:config_modules.tmp', 'w')
  fil_cfgmodout.writelines (at_moddata)
  fil_cfgmodout.close ()

  return 1

# def regcfg_reg_modules (t_p1_component, at_regdata, at_moddata):

# -----------------------------------------------------------------------------

def regcfg_unreg_modules (t_p1_component, at_p2_regdata, at_moddata):
  #++
  # unregister modules from (string)-list |at_p2_regdata|
  #
  # Returns
  #   1 = success, data in PYTHON_VMS:CONFIG_MODULES.TMP
  #  -1 = error
  #--

  # --------------------
  l_mod_idx = 0
  RemoveDone_Exceptn = 'RemoveDone'
  t_marker1 = '!$COMPONENT='    + t_p1_component
  t_marker2 = '!$ENDCOMPONENT=' + t_p1_component
  try:
    while (l_mod_idx < len (at_moddata)):
      # -- locate component
      if (at_moddata[l_mod_idx] [0:len(t_marker1)] == t_marker1):
        while (l_mod_idx < len (at_moddata)):
          # -- delete component
          if (at_moddata[l_mod_idx] [0:len(t_marker2)] == t_marker2):
            del (at_moddata[l_mod_idx]) # delete '!$ENDCOMPONENT'
            raise RemoveDone_Exceptn  # signal end to outer loops
          else:
            del (at_moddata[l_mod_idx]) # don't increment l_mod_idx!
        else:
          # reached EOF
          print '%regcfg_unreg_modules-f-bugchk, missing !$ENDCOMPONENT marker'
          import sys; sys.exit(0x2C)
      else:
        l_mod_idx = l_mod_idx + 1
    else:
      # reached EOF
      print '%regcfg_unreg_modules-f-bugchk, missing !$COMPONENT marker'
      import sys; sys.exit(0x2C)
  except RemoveDone_Exceptn:
    pass

  # --------------------
  # write new data
  fil_cfgmodout = open ('python_vms:config_modules.tmp', 'w')
  fil_cfgmodout.writelines (at_moddata)
  fil_cfgmodout.close ()

  return 1

# def regcfg_unreg_modules (t_p1_component, at_regdata, at_moddata):

# -----------------------------------------------------------------------------

def regcfg_reg_olb (t_p1_component, at_p2_regdata, at_olbdata):
  #++
  # register olb from (string)-list |at_p2_regdata|
  #
  # Returns
  #   1 = success, data in PYTHON_VMS:CONFIG_OLB.TMP
  #  -1 = error
  #--

  # locate EOF marker
  l_olb_idx = 0
  CopyDone_Exceptn = 'CopyDone'
  try:
    while (l_olb_idx < len (at_olbdata)):
      if (at_olbdata[l_olb_idx] [0:6] == '!$EOF='):
        # insert here
        l_reg_idx = 0
        # find begin of OLB data
        while (l_reg_idx < len (at_p2_regdata)):
          if (at_p2_regdata[l_reg_idx] [0:4] == '$OLB'):
            # found, insert
            l_reg_idx = l_reg_idx + 1
            at_olbdata.insert (l_olb_idx, '!$COMPONENT='+t_p1_component+'\n')
            l_olb_idx = l_olb_idx + 1
            while (l_reg_idx < len (at_p2_regdata)):
              if (at_p2_regdata[l_reg_idx] [0:7] == '$ENDOLB'):
                at_olbdata.insert (l_olb_idx, '!$ENDCOMPONENT='+t_p1_component+'\n')
                raise CopyDone_Exceptn  # signal end to outer loops
                break
              else:
                at_olbdata.insert (l_olb_idx, at_p2_regdata[l_reg_idx])
                l_olb_idx = l_olb_idx + 1
                l_reg_idx = l_reg_idx + 1
            else:
              print '%regcfg_reg_olb-f-bugchk, missing $ENDOLB marker'
              import sys; sys.exit(0x2C)
          l_reg_idx = l_reg_idx + 1
        else:
          print '%regcfg_reg_olb-f-bugchk, missing $OLB marker'
          import sys; sys.exit(0x2C)
        #-while
        break
      l_olb_idx = l_olb_idx + 1
    else:
      print '%regcfg_reg_olb-f-bugchk, missing !$EOF marker'
      import sys; sys.exit(0x2C)
  except CopyDone_Exceptn:
    pass

  # write new data
  fil_cfgolbout = open ('python_vms:config_olb.tmp', 'w')
  fil_cfgolbout.writelines (at_olbdata)
  fil_cfgolbout.close ()

  return 1

# def regcfg_reg_olb (t_p1_component, at_regdata, at_olbdata):

# -----------------------------------------------------------------------------

def regcfg_unreg_olb (t_p1_component, at_p2_regdata, at_olbdata):
  #++
  # unregister olb from (string)-list |at_p2_regdata|
  #
  # Returns
  #   1 = success, data in PYTHON_VMS:CONFIG_OLB.TMP
  #  -1 = error
  #--

  # --------------------
  l_olb_idx = 0
  RemoveDone_Exceptn = 'RemoveDone'
  t_marker1 = '!$COMPONENT='    + t_p1_component
  t_marker2 = '!$ENDCOMPONENT=' + t_p1_component
  try:
    while (l_olb_idx < len (at_olbdata)):
      # -- locate component
      if (at_olbdata[l_olb_idx] [0:len(t_marker1)] == t_marker1):
        while (l_olb_idx < len (at_olbdata)):
          # -- delete component
          if (at_olbdata[l_olb_idx] [0:len(t_marker2)] == t_marker2):
            del (at_olbdata[l_olb_idx]) # delete '!$ENDCOMPONENT'
            raise RemoveDone_Exceptn  # signal end to outer loops
          else:
            del (at_olbdata[l_olb_idx]) # don't increment l_olb_idx!
        else:
          # reached EOF
          print '%regcfg_unreg_modules-f-bugchk, missing !$ENDCOMPONENT marker'
          import sys; sys.exit(0x2C)
      else:
        l_olb_idx = l_olb_idx + 1
    else:
      # reached EOF
      print '%regcfg_unreg_modules-f-bugchk, missing !$COMPONENT marker'
      import sys; sys.exit(0x2C)
  except RemoveDone_Exceptn:
    pass

  # --------------------
  # write new data
  fil_cfgolbout = open ('python_vms:config_olb.tmp', 'w')
  fil_cfgolbout.writelines (at_olbdata)
  fil_cfgolbout.close ()

  return 1

# def regcfg_unreg_olb (t_p1_component, at_regdata, at_olbdata):

# -----------------------------------------------------------------------------

def regcfg_reg_config (t_p1_component, at_p2_regdata, at_cfgdata):
  #++
  # register modules from (string)-list |at_p2_regdata|
  #
  # Returns
  #   1 = success, data in PYTHON_VMS:CONFIG_DAT.TMP
  #  -1 = error
  #--

  # --------------------
  # -- locate EOF marker
  l_cfg_idx = 0
  CopyDone_Exceptn = 'CopyDone'
  try:
    while (l_cfg_idx < len (at_cfgdata)):
      if (at_cfgdata[l_cfg_idx] [0:21] == '!$CONFIGMODULE_MARKER'):
        # insert here
        l_reg_idx = 0
        # find begin of modules data
        while (l_reg_idx < len (at_p2_regdata)):
          if (at_p2_regdata[l_reg_idx] [0:7] == '$CONFIG'):
            # found, insert
            l_reg_idx = l_reg_idx + 1
            at_cfgdata.insert (l_cfg_idx, '!$COMPONENT='+t_p1_component+'\n')
            l_cfg_idx = l_cfg_idx + 1
            while (l_reg_idx < len (at_p2_regdata)):
              if (at_p2_regdata[l_reg_idx] [0:10] == '$ENDCONFIG'):
                at_cfgdata.insert (l_cfg_idx, '!$ENDCOMPONENT='+t_p1_component+'\n')
                raise CopyDone_Exceptn  # signal end to outer loops
                break
              else:
                at_cfgdata.insert (l_cfg_idx, at_p2_regdata[l_reg_idx])
                l_cfg_idx = l_cfg_idx + 1
                l_reg_idx = l_reg_idx + 1
            else:
              print '%regcfg_reg_config-f-bugchk, missing $ENDMODULES marker'
              import sys; sys.exit(0x2C)
          l_reg_idx = l_reg_idx + 1
        else:
          print '%regcfg_reg_config-f-bugchk, missing $MODULES marker'
          import sys; sys.exit(0x2C)
        #-while
        break
      l_cfg_idx = l_cfg_idx + 1
    else:
      print '%regcfg_reg_config-f-bugchk, missing !$EOF marker'
      import sys; sys.exit(0x2C)
  except CopyDone_Exceptn:
    pass

  # write new data
  fil_cfgcfgout = open ('python_vms:config_dat.tmp', 'w')
  fil_cfgcfgout.writelines (at_cfgdata)
  fil_cfgcfgout.close ()

  return 1

# def regcfg_reg_config (t_p1_component, at_p2_regdata, at_cfgdata):
# -----------------------------------------------------------------------------

def regcfg_unreg_config (t_p1_component, at_p2_regdata, at_cfgdata):
  #++
  # unregister modules from (string)-list |at_p2_regdata|
  #
  # Returns
  #   1 = success, data in PYTHON_VMS:CONFIG_DAT.TMP
  #  -1 = error
  #--

  # --------------------
  l_cfg_idx = 0
  RemoveDone_Exceptn = 'RemoveDone'
  t_marker1 = '!$COMPONENT='    + t_p1_component
  t_marker2 = '!$ENDCOMPONENT=' + t_p1_component
  try:
    while (l_cfg_idx < len (at_cfgdata)):
      # -- locate component
      if (at_cfgdata[l_cfg_idx] [0:len(t_marker1)] == t_marker1):
        while (l_cfg_idx < len (at_cfgdata)):
          # -- delete component
          if (at_cfgdata[l_cfg_idx] [0:len(t_marker2)] == t_marker2):
            del (at_cfgdata[l_cfg_idx]) # delete '!$ENDCOMPONENT'
            raise RemoveDone_Exceptn  # signal end to outer loops
          else:
            del (at_cfgdata[l_cfg_idx]) # don't increment l_cfg_idx!
        else:
          # reached EOF
          print '%regcfg_unreg_config-f-bugchk, missing !$ENDCOMPONENT marker'
          import sys; sys.exit(0x2C)
      else:
        l_cfg_idx = l_cfg_idx + 1
    else:
      # reached EOF
      print '%regcfg_unreg_config-f-bugchk, missing !$COMPONENT marker'
      import sys; sys.exit(0x2C)
  except RemoveDone_Exceptn:
    pass

  # --------------------
  # write new data
  fil_cfgmodout = open ('python_vms:config_dat.tmp', 'w')
  fil_cfgmodout.writelines (at_cfgdata)
  fil_cfgmodout.close ()

  return 1

# def regcfg_unreg_config (t_p1_component, at_p2_regdata, at_cfgdata):

# -----------------------------------------------------------------------------

def regcfg__check_cfgdata (t_p1_filespec):
  #++
  # load configuration data (update) from |t_p1_filespec| and validate
  #
  # Returns:
  #   at_regdata   = [list]   or None
  #   t_component  = "string" or None
  #   x_consistent = 0/1
  #--

  # --------------------
  # read whole file into memory
  try:
    fil_cfgregdat = open (t_p1_filespec,  'r')
  except IOError, x_exceptn:
    print '%REGISTER_CONFIG-F-OPENIN, error opening input file'
    print '-regcfg__check_cfgdata:', x_exceptn
    print ' cannot (un)register'
    return (None, None, -1)

  # load all data
  at_regdata = fil_cfgregdat.readlines ()
  fil_cfgregdat.close()

  # --------------------
  # make a consistency check
  l_fnd_component  = 0
  l_fnd_config     = 0
  l_fnd_endconfig  = 0
  l_fnd_modules    = 0
  l_fnd_endmodules = 0
  l_fnd_olb        = 0
  l_fnd_endolb     = 0
  #
  l_idx = 0
  while (l_idx < len (at_regdata)):
    if (at_regdata[l_idx] [0:11] == '$COMPONENT='):
      t_component = at_regdata[l_idx] [11:-1]   # :-1 = remove trailing \n
      l_fnd_component = l_fnd_component + 1
    elif (at_regdata[l_idx] [0:7] == '$CONFIG'):
      l_fnd_config = l_fnd_config + 1
    elif (at_regdata[l_idx] [0:10] == '$ENDCONFIG'):
      l_fnd_endconfig = l_fnd_endconfig + 1
    elif (at_regdata[l_idx] [0:8] == '$MODULES'):
      l_fnd_modules = l_fnd_modules + 1
    elif (at_regdata[l_idx] [0:11] == '$ENDMODULES'):
      l_fnd_endmodules = l_fnd_endmodules + 1
    elif (at_regdata[l_idx] [0:4] == '$OLB'):
      l_fnd_olb = l_fnd_olb + 1
    elif (at_regdata[l_idx] [0:7] == '$ENDOLB'):
      l_fnd_endolb = l_fnd_endolb + 1
    #
    l_idx = l_idx + 1                           # next line
  #-while
  # if everything is ok, then the component name is now in |t_component|

  # --------------------
  # collect error messages
  at_badlist = []

  if (l_fnd_component > 1):
    at_badlist.append (' multiple "$COMPONENT=" markers')
  elif (l_fnd_component == 0):
    at_badlist.append (' missing "$COMPONENT=" markers')

  if (l_fnd_config > 1):
    at_badlist.append (' multiple "$CONFIG" markers')
  elif (l_fnd_config == 0):
    at_badlist.append (' missing "$CONFIG" markers')

  if (l_fnd_endconfig > 1):
    at_badlist.append (' multiple "$ENDCONFIG" markers')
  elif (l_fnd_endconfig == 0):
    at_badlist.append (' missing "$ENDCONFIG" markers')

  if (l_fnd_modules > 1):
    at_badlist.append (' multiple "$MODULES" markers')
  elif (l_fnd_modules == 0):
    at_badlist.append (' missing "$MODULES" markers')

  if (l_fnd_endmodules > 1):
    at_badlist.append (' multiple "$ENDMODULES" markers')
  elif (l_fnd_endmodules == 0):
    at_badlist.append (' missing "$ENDMODULES" markers')

  if (l_fnd_olb > 1):
    at_badlist.append (' multiple "$OLB" markers')
  elif (l_fnd_olb == 0):
    at_badlist.append (' missing "$OLB" markers')

  if (l_fnd_endolb > 1):
    at_badlist.append (' multiple "$ENDOLB" markers')
  elif (l_fnd_endolb == 0):
    at_badlist.append (' missing "$ENDOLB" markers')

  # --------------------
  # print all error messages
  if (at_badlist):
    print '%REGISTER_CONFIG-F-REGDATBAD, registration data bad'
    for t_txt in at_badlist:
      print t_txt
    return (None, None, -1)

  return (at_regdata, t_component, 1)

# def regcfg__check_cfgdata (t_p1_filespec):

# -----------------------------------------------------------------------------

def regcfg__load_python_config ():
  #++
  # load Python configuration data files
  #   - PYTHON_VMS:CONFIG.DAT
  #   - PYTHON_VMS:CONFIG_MODULES.TXT
  #   - PYTHON_VMS:CONFIG_OLB.TXT
  #
  # Returns
  #   status     = 1, -1
  #   at_cfgdata = [list]
  #   at_moddata = [list]
  #   at_olbdata = [list]
  #--

  # --------------------
  # load PYTHON_VMS:CONFIG.DAT
  try:
    fil_cfgcfgfnd = open ('python_vms:config.dat', 'r')
  except IOError, x_exceptn:
    print '%REGISTER_CONFIG-F-OPENIN, error opening input file'
    print '-regcfg__load_python_config:', x_exceptn
    print ' you NEED this file for building Python!'
    return (-1, None, None)

  # load all data
  at_cfgdata = fil_cfgcfgfnd.readlines ()
  fil_cfgcfgfnd.close()

  # --------------------
  # load PYTHON_VMS:CONFIG_MODULES.TXT
  try:
    fil_cfgmodfnd = open ('python_vms:config_modules.txt', 'r')
  except IOError, x_exceptn:
    print '%REGISTER_CONFIG-F-OPENIN, error opening input file'
    print '-regcfg__load_python_config:', x_exceptn
    print ' you NEED this file for building Python!'
    return (-1, None, None)

  # load all data
  at_moddata = fil_cfgmodfnd.readlines ()
  fil_cfgmodfnd.close()

  # --------------------
  # load PYTHON_VMS:CONFIG_OLB.TXT
  try:
    fil_cfgolbfnd = open ('python_vms:config_olb.txt', 'r')
  except IOError, x_exceptn:
    print '%REGISTER_CONFIG-F-OPENIN, error opening input file'
    print '-regcfg__load_python_config:', x_exceptn
    print ' you NEED this file for building Python!'
    return (-1, None, None)

  # load all data
  at_olbdata = fil_cfgolbfnd.readlines ()
  fil_cfgolbfnd.close()

  # --------------------
  return (1, at_cfgdata, at_moddata, at_olbdata)

# def regcfg__load_python_config ():

# -----------------------------------------------------------------------------

def regcfg__register (t_p1_filespec):
  #++
  # register components from file |t_p1_filespec|
  #
  # Returns
  #     -1 = error
  #      1 = registered
  #--

  # --------------------
  # validate registration data and return in-memory list
  at_regdata, t_component, x_ok = regcfg__check_cfgdata (t_p1_filespec)
  if (x_ok <> 1):
    return -1

  # --------------------
  # load Python configuration data files (CONFIG_MODULES.TXT + CONFIG_OLB.TXT
  status, at_cfgdata, at_moddata, at_olbdata = regcfg__load_python_config ()
  if (status <> 1):
    return -1

  # --------------------
  # check whether this component has already been registered
  x_found = regcfg__find_component_in_config \
		(t_component, at_moddata, at_olbdata, at_cfgdata)

  regmod_status = 0
  regolb_status = 0
  regcfg_status = 0
  if (x_found == 0):
    # @@ start register process
    regmod_status = regcfg_reg_modules (t_component, at_regdata, at_moddata)
    if (regmod_status):
      regolb_status = regcfg_reg_olb (t_component, at_regdata, at_olbdata)
      if (regolb_status):
        regcfg_status = regcfg_reg_config (t_component, at_regdata, at_cfgdata)

    if ((regmod_status == 1) and (regolb_status == 1) and
        (regcfg_status == 1)                              ):
      import os
      os.rename ('python_vms:config_modules.tmp',
                 'python_vms:config_modules.txt;0')
      os.rename ('python_vms:config_olb.tmp',
                 'python_vms:config_olb.txt;0')
      os.rename ('python_vms:config_dat.tmp',
                 'python_vms:config.dat;0')
      print '%%REGISTER_CONFIG-S-COMPREG, component "%s" registered' % \
            (t_component,)
      return 1
    else:
      return -1
  else:
    print '%%REGISTER_CONFIG-I-COMPALREG, component "%s" already registered' % \
          (t_component,)
  # --------------------
  return -1

# def regcfg__register (t_p1_filespec):

# -----------------------------------------------------------------------------

def regcfg__unregister (t_p1_filespec):
  #++
  # unregister components from file |t_p1_filespec|
  #
  # Returns
  #     -1 = error
  #      1 = unregistered
  #--

  # --------------------
  # validate registration data and return in-memory list
  at_regdata, t_component, x_ok = regcfg__check_cfgdata (t_p1_filespec)
  if (x_ok <> 1):
    return -1

  # --------------------
  # load Python configuration data files (CONFIG_MODULES.TXT + CONFIG_OLB.TXT
  status, at_cfgdata, at_moddata, at_olbdata = regcfg__load_python_config ()
  if (status <> 1):
    return -1

  # --------------------
  # check whether this component has been registered
  x_found = regcfg__find_component_in_config \
		(t_component, at_moddata, at_olbdata, at_cfgdata)

  unregmod_status = 0
  unregolb_status = 0
  unregcfg_status = 0
  if (x_found == 1):
    # start unregister process
    unregmod_status = regcfg_unreg_modules (t_component, at_regdata, at_moddata)
    if (unregmod_status):
      unregolb_status = regcfg_unreg_olb (t_component, at_regdata, at_olbdata)
      if (unregolb_status):
        unregcfg_status = regcfg_unreg_config (t_component, at_regdata, at_cfgdata)

    if ((unregmod_status == 1) and (unregolb_status == 1) and
        (unregcfg_status == 1)                               ):
      import os
      os.rename ('python_vms:config_modules.tmp',
                 'python_vms:config_modules.txt;0')
      os.rename ('python_vms:config_olb.tmp',
                 'python_vms:config_olb.txt;0')
      os.rename ('python_vms:config_dat.tmp',
                 'python_vms:config.dat;0')
      print '%%REGISTER_CONFIG-S-COMPUNREG, component "%s" unregistered' % \
            (t_component,)
      return 1
    else:
      return -1
  else:
    print '%%REGISTER_CONFIG-E-COMPNOTREG, component "%s" not registered' % \
          (t_component,)
  # --------------------
  return -1

# def regcfg__unregister (t_p1_filespec):

# -----------------------------------------------------------------------------

if __name__ == '__main__':
    import sys

    if (len(sys.argv) <> 3):
        print '%REGISTER_CONFIG-F-IVARG, invalid number of arguments - need 2'
        sys.exit(1)

    t_mode     = sys.argv [1]
    t_filespec = sys.argv [2]

    if ((t_mode == 'R') or (t_mode == 'r')):
        x_status = regcfg__register (t_filespec)
    else:
        if ((t_mode == 'U') or (t_mode == 'u')):
            x_status = regcfg__unregister (t_filespec)
        else:
            print '%REGISTER_CONFIG-F-IVPARM, parameter 1 invalid - use R or U'
            sys.exit(1)
# --------------------

# -----------------------------------------------------------------------------
