/* VMS__LOCALTIME.C -- localtime() replacement -- 24-FEB-2000 Uwe Zessin

convert a calendar time to a structure of type tm.

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS__LOCALTIME

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS__LOCALTIME "20000226"
#endif

/* ------------------------------------------------------------------------- */
#include <time.h>
#include <stdio.h>

#include <lib$routines.h>
#include <starlet.h>

/* ------------------------------------------------------------------------- */
void vms__tzset (void);

/* in VMS__CVT.C */
void vms__cvt_u2v_time (time_t al_unixtime, unsigned long *aq_vmstime);

/* in PYVMS_GLOBAL.C */
extern int		vms_gl_daylight;
extern long		vms_gl_timezone;

extern long		vms_gl_tzdiff;

struct tm		vms_gr_lcltim_tm;

/* ------------------------------------------------------------------------- */

struct tm * vms__localtime (time_t * timer)
{
	time_t			local_timer;
	long			l_tzdif;		/* signed value ! */

	unsigned long		q_bintim [2];
	unsigned short int	w_timbuf[7];
#define K_YEAR_SINCE_0		0
#define K_MONTH_OF_YEAR		1
#define K_DAY_OF_MONTH		2
#define K_HOUR_OF_DAY		3
#define K_MINUTE_OF_HOUR	4
#define K_SECOND_OF_MINUTE	5
#define K_HUNDREDTHS_OF_SECOND	6

	long			l_week_day;

	unsigned long		l_status;

	int	l_days_in_months[] = {31,28,31,30,31,30,31,31,30,31,30,31};
	int	l_loop;
	int	l_doy;

	/* -------------------- */
	(void) vms__tzset ();

	/* -------------------- */
	/* make (simple) adjustments for DST */
	if (vms_gl_daylight)	/* system is in an area with DST handling */
	{
	    /* apply timezone difference */
	    if (vms_gl_tzdiff < 0)
	    {
		/* being west of GMT - subtract difference, but: */
		/* prevent underrun (a date before 1-JAN-1970) */
		if (*timer > (-vms_gl_tzdiff))
		{
		    /* remember that |vms_gl_tzdiff| is negative */
		    local_timer = *timer + vms_gl_tzdiff;
		}
		else
		{
		    /* would unterrun - use: */
		    /* 1-JAN-1970 - the earliest date possible */
		    local_timer = 0;
		}			
	    }
	    else
	    {
		/* being east of GMT - add difference */
		local_timer = *timer + vms_gl_tzdiff;
	    }
	}
	else
	{
	    local_timer = *timer;
	}

	/* -------------------- */
	/* convert to OpenVMS binary date for use by $NUMTIM */
	(void) vms__cvt_u2v_time (local_timer, &q_bintim [0]);

	/* -------------------- */
	/* @@ SYS$NUMTIM might not be available in OpenVMS V5.5-2 */
	l_status = sys$numtim
		(&w_timbuf[0]		/* timbuf   */
		,&q_bintim [0]		/* [timadr] */
		);

	/* -------------------- */
        vms_gr_lcltim_tm.tm_sec   = w_timbuf[K_SECOND_OF_MINUTE]; /* seconds after the minute [0, 61] */
        vms_gr_lcltim_tm.tm_min   = w_timbuf[K_MINUTE_OF_HOUR];   /* minutes after the hour [0, 59] */
        vms_gr_lcltim_tm.tm_hour  = w_timbuf[K_HOUR_OF_DAY];      /* hours since midnight [0, 23] */
        vms_gr_lcltim_tm.tm_mday  = w_timbuf[K_DAY_OF_MONTH];     /* day of the month [1, 31] */
        vms_gr_lcltim_tm.tm_mon   = w_timbuf[K_MONTH_OF_YEAR] -1; /* months since January [0, 11] */
        vms_gr_lcltim_tm.tm_year  = w_timbuf[K_YEAR_SINCE_0];     /* years since 1900 */
	vms_gr_lcltim_tm.tm_year -= 1900;	/* sys$numtim returns full year */

	/* -------------------- */
	l_status = lib$day_of_week
		(&q_bintim [0]
		,&l_week_day		/* day of week, 1=Mon, 7=Sun */
		);
	if (l_week_day == 7)
	{
	    l_week_day = 0;
	}
        vms_gr_lcltim_tm.tm_wday = l_week_day;	/* days since Sunday [0, 6] */

	/* -------------------- */
	/* @@ this breaks in year 2100 which is not a leap year */
	if ((w_timbuf[K_YEAR_SINCE_0] % 4) == 0)
	{
	    l_days_in_months[1] = 29;
	}
	l_doy = 0;
	for (l_loop = 0; l_loop < (w_timbuf[K_MONTH_OF_YEAR]-1); l_loop++)
	{
	    l_doy += l_days_in_months[l_loop];
	}
	l_doy += (w_timbuf[K_DAY_OF_MONTH]-1);
        vms_gr_lcltim_tm.tm_yday = l_doy;	/* days since January 1 [0, 365] */

	/* -------------------- */
        vms_gr_lcltim_tm.tm_isdst = 0;		/* Daylight Saving Time flag */
	if (vms_gl_daylight)	/* system is in an area with DST handling */
	{
	    if (vms_gl_timezone != vms_gl_tzdiff)
	    {
		/* |vms_gl_timezone| is the offset to UTC during non-DST */
		/* |vms_gl_tzdiff| is the current value from the logical */
		/*                 name SYS$TIMEZONE_DIFFERENTIAL */
		/* if the values are different, DST is currently in effect */
		vms_gr_lcltim_tm.tm_isdst = 1;
	    }
	}

	/* -------------------- */
	return (&vms_gr_lcltim_tm);

} /* struct tm * vms__localtime (time_t * timer) */

/* ------------------------------------------------------------------------- */

/* EOF: VMS__LOCALTIME.C */
