/* VMS_SMG_CONTROL_MODE.C -- 05-MAR-2000 Uwe Zessin
   Python interface to SMG$CONTROL_MODE

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_CONTROL_MODE

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_CONTROL_MODE "20000305"
#endif

/* ------------------------------------------------------------------------- */

#include "python.h"
#include "vmsdef.h"

#include <smg$routines.h>	/* SMG$name  */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_control_mode__doc[] =
"old_mode = vms_smg.control_mode (pasteboard_id, [new_mode], [buffer_size])\n\
Control Mode.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_control_mode (PyObject *self, PyObject *args)
{
	unsigned long		  l_pasteboard_id;

	PyObject		* ar_new_mode;
	unsigned long		  l_new_mode;
	unsigned long		* al_new_mode;

	unsigned long		  l_old_mode;	

	PyObject		* ar_buffer_size;
	unsigned long		  l_buffer_size;
	unsigned short int	  w_buffer_size;
	unsigned short int	* aw_buffer_size;

	unsigned long		  l_status;

	/* -------------------- */
	ar_new_mode      = Py_None;
	ar_buffer_size   = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|OO"
	    ,&l_pasteboard_id
	    ,&ar_new_mode
	    ,&ar_buffer_size
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: pasteboard_id */

	/* -------------------- */
	/* argument 2: [new_mode] */
	if (ar_new_mode == Py_None)
	{
	    al_new_mode = 0;		/* omitted */
	    l_new_mode  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_new_mode))
	    {
		PyErr_SetString(PyExc_TypeError,
		   "argument 2: new_mode - must be integer or None");
		return NULL;
	    }
	    l_new_mode = PyInt_AsLong(ar_new_mode);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_new_mode = &l_new_mode;
	}

	/* -------------------- */
	/* argument 3: [buffer_size] */
	if (ar_buffer_size == Py_None)
	{
	    aw_buffer_size = 0;		/* omitted */
	    w_buffer_size  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_buffer_size))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: buffer_size - must be integer or None");
		return NULL;
	    }
	    l_buffer_size = PyInt_AsLong(ar_buffer_size);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    if ((l_buffer_size < 0) || (l_buffer_size > 65535))
	    {
		PyErr_SetString(PyExc_ValueError,
		    "argument 3: buffer_size - must be between 256 and 65535");
		return NULL;
	    }
	    w_buffer_size  = l_buffer_size;
	    aw_buffer_size = &w_buffer_size;
	}

	/* -------------------- */
	l_old_mode = 0;

	/* -------------------- */
	l_status = smg$control_mode
		(&l_pasteboard_id
		,al_new_mode
		,&l_old_mode
		,aw_buffer_size
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    return Py_BuildValue ("i", l_old_mode);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_control_mode () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_CONTROL_MODE.C */
