/* VMS_SMG_DRAW_CHAR.C -- 06-MAR-2000 Uwe Zessin
   Python interface to SMG$DRAW_CHAR

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_DRAW_CHAR

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_DRAW_CHAR "20000306"
#endif

/* ------------------------------------------------------------------------- */

#include "python.h"
#include "vmsdef.h"

#include <smg$routines.h>	/* SMG$name  */
#include <ssdef.h>		/* SS$_name  */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_draw_char__doc[] =
"None = vms_smg.draw_char (display_id, flags, [row], [column],\
 [rendition_set], [rendition_complement])\n\
Draw a Character in a Virtual Display.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_draw_char (PyObject *self, PyObject *args)
{
	unsigned long		  l_display_id;
	unsigned long		  l_flags;

	PyObject		* ar_row;
	long			  l_row;
	long			* al_row;

	PyObject		* ar_column;
	long			  l_column;
	long			* al_column;

	PyObject		* ar_rendition_set;
	unsigned long		  l_rendition_set;
	unsigned long		* al_rendition_set;

	PyObject		* ar_rendition_complement;
	unsigned long		  l_rendition_complement;
	unsigned long		* al_rendition_complement;

	unsigned long		  l_status;

	/* -------------------- */
	ar_row                  = Py_None;
	ar_column               = Py_None;
	ar_rendition_set        = Py_None;
	ar_rendition_complement = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "ii|OOOO"
	    ,&l_display_id
	    ,&l_flags
	    ,&ar_row
	    ,&ar_column
	    ,&ar_rendition_set
	    ,&ar_rendition_complement
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */
	/* -------------------- */
	/* argument 2: flags */

	/* -------------------- */
	/* argument 3: [row] */
	if (ar_row == Py_None)
	{
	    al_row = 0;		/* omitted */
	    l_row  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_row))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: row - must be integer or None");
		return NULL;
	    }
	    l_row = PyInt_AsLong(ar_row);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_row = &l_row;
	}

	/* -------------------- */
	/* argument 4: [column] */
	if (ar_column == Py_None)
	{
	    al_column = 0;		/* omitted */
	    l_column  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_column))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 4: column - must be integer or None");
		return NULL;
	    }
	    l_column = PyInt_AsLong(ar_column);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_column = &l_column;
	}

	/* -------------------- */
	/* argument 5: [rendition_set] */
	if (ar_rendition_set == Py_None)
	{
	    al_rendition_set = 0;		/* omitted */
	    l_rendition_set  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_set))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 5: rendition_set - must be integer or None");
		return NULL;
	    }
	    l_rendition_set = PyInt_AsLong(ar_rendition_set);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_set = &l_rendition_set;
	}

	/* -------------------- */
	/* argument 6: [rendition_complement] */
	if (ar_rendition_complement == Py_None)
	{
	    al_rendition_complement = 0;		/* omitted */
	    l_rendition_complement  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_rendition_complement))
	    {
		PyErr_SetString(PyExc_TypeError,
		 "argument 6: rendition_complement - must be integer or None");
		return NULL;
	    }
	    l_rendition_complement = PyInt_AsLong(ar_rendition_complement);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_rendition_complement = &l_rendition_complement;
	}

	/* -------------------- */
	l_status = smg$draw_char
		(&l_display_id
		,&l_flags
		,al_row
		,al_column
		,al_rendition_set
		,al_rendition_complement
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_draw_char () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_DRAW_CHAR.C */
