/* VMS_SMG_READ_FROM_DISPLAY.C -- 25-MAR-2000 Uwe Zessin
   Python interface to SMG$READ_FROM_DISPLAY

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_READ_FROM_DISPLAY

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_READ_FROM_DISPLAY "20000325"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <lib$routines.h>	/* LIB$name */
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_read_from_display__doc[] =
"status, resultant_string = vms_smg.read_from_display\
 (display_id, [terminator_string], [start_row], [rendition_string])\n\
Read Text from Display.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_read_from_display (PyObject *self, PyObject *args)
{
	unsigned long		  l_display_id;

	char			* at_terminator_string;
	struct dsc$descriptor_s   r_terminator_string;
	struct dsc$descriptor_s * ar_terminator_string;
	unsigned long		  l_terminator_string_len;

	PyObject		* ar_start_row;
	unsigned long		  l_start_row;
	unsigned long		* al_start_row;

	char			* at_rendition_string;
	struct dsc$descriptor_s   r_rendition_string;
	struct dsc$descriptor_s * ar_rendition_string;
	unsigned long		  l_rendition_string_len;

	struct dsc$descriptor_s   r_resultant_string;
	unsigned short		  w_resultant_length;

	unsigned long		  l_status_smg;
	unsigned long		  l_status_free;

	/* -------------------- */
	at_terminator_string = NULL;    l_terminator_string_len = 0;
	ar_start_row         = Py_None;
	at_rendition_string  = NULL;    l_rendition_string_len = 0;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|z#Oz#"
	    ,&l_display_id
	    ,&at_terminator_string ,&l_terminator_string_len
	    ,&ar_start_row
	    ,&at_rendition_string  ,&l_rendition_string_len
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */

	/* -------------------- */
	/* argument 2: [terminator_string} */
	if (at_terminator_string == NULL)
	{
	    ar_terminator_string = 0;	/* omitted */
	}
	else
	{
	    if (l_terminator_string_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
   "argument 2: terminator_string - string size limited to 65535 characters");
		return NULL;
	    }
	    r_terminator_string.dsc$w_length  = l_terminator_string_len;
	    r_terminator_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_terminator_string.dsc$b_class   = DSC$K_CLASS_S;
	    r_terminator_string.dsc$a_pointer = at_terminator_string;
	    ar_terminator_string = &r_terminator_string;
	}

	/* -------------------- */
	/* argument 3: [start_row] */
	if (ar_start_row == Py_None)
	{
	    al_start_row = 0;		/* omitted */
	    l_start_row  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_start_row))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: start_row - must be integer or None");
		return NULL;
	    }
	    l_start_row = PyInt_AsLong(ar_start_row);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_start_row = &l_start_row;
	}

	/* -------------------- */
	/* argument 4: [rendition_string} */
	if (at_rendition_string == NULL)
	{
	    ar_rendition_string = 0;	/* omitted */
	}
	else
	{
	    if (l_rendition_string_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
    "argument 4: rendition_string - string size limited to 65535 characters");
		return NULL;
	    }
	    r_rendition_string.dsc$w_length  = l_rendition_string_len;
	    r_rendition_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_rendition_string.dsc$b_class   = DSC$K_CLASS_S;
	    r_rendition_string.dsc$a_pointer = at_rendition_string;
	    ar_rendition_string = &r_rendition_string;
	}

	/* -------------------- */
	w_resultant_length = 0;
	/* set up string descriptor */
	/*  let SMG$READ_FROM_DISPLAY allocate the memory */
	r_resultant_string.dsc$w_length  = 0;
	r_resultant_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_resultant_string.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_resultant_string.dsc$a_pointer = 0;

	/* -------------------- */
	l_status_smg = smg$read_from_display
		(&l_display_id
		,&r_resultant_string
		,ar_terminator_string
		,al_start_row
		,ar_rendition_string
		);

	/* -------------------- */
	{
	    PyObject	* ar_return;

	    ar_return = Py_BuildValue
		("is#"
		,l_status_smg
		,(r_resultant_string.dsc$w_length == 0)	/* empty string? */
		  ? (void*)&r_resultant_string		/* non-NULL address */
		  : (void*)r_resultant_string.dsc$a_pointer
		,(unsigned int)r_resultant_string.dsc$w_length
		);

	    /* -------------------- */
	    /* deallocate memory of dynamic string descriptor */
	    l_status_free = lib$sfree1_dd (&r_resultant_string);
	    if (l_status_free != SS$_NORMAL)
	    {
		PyErr_SetString(PyExc_SystemError,
		    "vms_smg_read_from_display: LIB$SFREE1_DD() failed");
		(void) PyErr_Print();

		Py_XDECREF(ar_return);

		/* -------------------- */
		/* error */
		return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status_free);
	    }

	    /* -------------------- */
	    return ar_return;
	}

} /* vms_smg_read_from_display () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_READ_FROM_DISPLAY.C */
