/* VMS_SMG_RETURN_INPUT_LINE.C -- 01-APR-2000 Uwe Zessin
   Python interface to SMG$RETURN_INPUT_LINE

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_RETURN_INPUT_LINE

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_RETURN_INPUT_LINE "20000401"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <descrip.h>
#include <lib$routines.h>	/* LIB$name */
#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_return_input_line__doc[] =
"status, resultant_string = vms_smg.return_input_line\
 (keyboard_id, [match_string], [line_number])\n\
Return Input Line.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_return_input_line (PyObject *self, PyObject *args)
{
	unsigned long		  l_keyboard_id;

	char			* at_match_string;
	struct dsc$descriptor_s   r_match_string;
	struct dsc$descriptor_s * ar_match_string;
	unsigned long		  l_match_string_len;

	PyObject		* ar_line_number;
	long			  l_line_number;
	long			* al_line_number;

	struct dsc$descriptor_s   r_resultant_string;
	unsigned short		  w_resultant_length;

	unsigned long		  l_status_smg;
	unsigned long		  l_status_free;

	/* -------------------- */
	at_match_string = NULL;   l_match_string_len = 0;
	ar_line_number  = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|z#O"
	    ,&l_keyboard_id
	    ,&at_match_string       ,&l_match_string_len
	    ,&ar_line_number
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: keyboard_id */

	/* -------------------- */
	/* argument 2: [match_string} */
	if (at_match_string == NULL)
	{
	    ar_match_string = 0;	/* omitted */
	}
	else
	{
	    if (l_match_string_len > 65535)
	    {
		PyErr_SetString(PyExc_ValueError,
       "argument 2: match_string - string size limited to 65535 characters");
		return NULL;
	    }
	    r_match_string.dsc$w_length  = l_match_string_len;
	    r_match_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	    r_match_string.dsc$b_class   = DSC$K_CLASS_S;
	    r_match_string.dsc$a_pointer = at_match_string;
	    ar_match_string = &r_match_string;
	}

	/* -------------------- */
	/* argument 3: [line_number] */
	if (ar_line_number == Py_None)
	{
	    al_line_number = 0;		/* omitted */
	    l_line_number  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_line_number))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: line_number - must be integer or None");
		return NULL;
	    }
	    l_line_number = PyInt_AsLong(ar_line_number);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_line_number = &l_line_number;
	}

	/* -------------------- */
	/* set up string descriptor */
	/*  let SMG$RETURN_INPUT_LINE allocate the memory */
	r_resultant_string.dsc$w_length  = 0;
	r_resultant_string.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_resultant_string.dsc$b_class   = DSC$K_CLASS_D; /* not _S !! */
	r_resultant_string.dsc$a_pointer = 0;

	w_resultant_length = 0;

	/* -------------------- */
	l_status_smg = smg$return_input_line
		(&l_keyboard_id
		,&r_resultant_string
		,ar_match_string
		,al_line_number
		,&w_resultant_length
		);

	/* -------------------- */
	{
	    PyObject	* ar_return;

	    ar_return = Py_BuildValue
		("is#"
		,l_status_smg

		,(r_resultant_string.dsc$w_length == 0)	/* empty string? */
		  ? (void*)&r_resultant_string		/* non-NULL address */
		  : (void*)r_resultant_string.dsc$a_pointer
		,(unsigned int)r_resultant_string.dsc$w_length
		);

	    /* -------------------- */
	    /* deallocate memory of dynamic string descriptor */
	    l_status_free = lib$sfree1_dd (&r_resultant_string);

	    if (l_status_free != SS$_NORMAL)
	    {
		PyErr_SetString(PyExc_SystemError,
		    "vms_smg_return_input_line: LIB$SFREE1_DD() failed");
		(void) PyErr_Print();

		Py_XDECREF(ar_return);

		/* -------------------- */
		/* error */
		return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status_free);
	    }

	    /* -------------------- */
	    return ar_return;
	}
} /* vms_smg_return_input_line () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_RETURN_INPUT_LINE.C */
