/* VMS_SMG_SCROLL_DISPLAY_AREA.C -- 01-APR-2000 Uwe Zessin
   Python interface to SMG$SCROLL_DISPLAY_AREA

--------------------
$ @ PYTHON_VMS:BLDRUN  VMS  VMS_SMG_SCROLL_DISPLAY_AREA

*/

#if defined(__DECC) || defined(__DECCXX)
#pragma module VMS_SMG_SCROLL_DISPLAY_AREA "20000401"
#endif

/* ------------------------------------------------------------------------- */

#include "vmsdef.h"		/* includes "Python.h" */

#include <smg$routines.h>	/* SMG$name */
#include <ssdef.h>		/* SS$_name */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_smg_gr_error;		/* exception vms_smg.error */
/* ------------------------------------------------------------------------- */

char vms_smg_scroll_dpy_area__doc[] =
"None = vms_smg.scroll_display_area (display_id, [start_row], [start_column],\
 [height], [width], [direction], [count])\n\
Scroll Display Area.";

/* ------------------------------------------------------------------------- */

PyObject *
vms_smg_scroll_display_area (PyObject *self, PyObject *args)
{
	unsigned long		  l_display_id;

	PyObject		* ar_start_row;
	long			  l_start_row;
	long			* al_start_row;

	PyObject		* ar_start_column;
	long			  l_start_column;
	long			* al_start_column;

	PyObject		* ar_height;
	long			  l_height;
	long			* al_height;

	PyObject		* ar_width;
	long			  l_width;
	long			* al_width;

	PyObject		* ar_direction;
	unsigned long		  l_direction;
	unsigned long		* al_direction;

	PyObject		* ar_count;
	long			  l_count;
	long			* al_count;

	unsigned long		  l_status;

	/* -------------------- */
	ar_start_row    = Py_None;
	ar_start_column = Py_None;
	ar_height       = Py_None;
	ar_width        = Py_None;
	ar_direction    = Py_None;
	ar_count        = Py_None;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "i|OOOOOO"
	    ,&l_display_id
	    ,&ar_start_row
	    ,&ar_start_column
	    ,&ar_height
	    ,&ar_width
	    ,&ar_direction
	    ,&ar_count
	    ))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: display_id */

	/* -------------------- */
	/* argument 2: [start_row] */
	if (ar_start_row == Py_None)
	{
	    al_start_row = 0;		/* omitted */
	    l_start_row  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_start_row))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 2: start_row - must be integer or None");
		return NULL;
	    }
	    l_start_row = PyInt_AsLong(ar_start_row);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_start_row = &l_start_row;
	}

	/* -------------------- */
	/* argument 3: [start_column] */
	if (ar_start_column == Py_None)
	{
	    al_start_column = 0;		/* omitted */
	    l_start_column  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_start_column))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 3: start_column - must be integer or None");
		return NULL;
	    }
	    l_start_column = PyInt_AsLong(ar_start_column);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_start_column = &l_start_column;
	}

	/* -------------------- */
	/* argument 4: [height] */
	if (ar_height == Py_None)
	{
	    al_height = 0;		/* omitted */
	    l_height  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_height))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 4: height - must be integer or None");
		return NULL;
	    }
	    l_height = PyInt_AsLong(ar_height);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_height = &l_height;
	}

	/* -------------------- */
	/* argument 5: [width] */
	if (ar_width == Py_None)
	{
	    al_width = 0;		/* omitted */
	    l_width  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_width))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 5: width - must be integer or None");
		return NULL;
	    }
	    l_width = PyInt_AsLong(ar_width);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_width = &l_width;
	}

	/* -------------------- */
	/* argument 6: [direction] */
	if (ar_direction == Py_None)
	{
	    al_direction = 0;		/* omitted */
	    l_direction  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_direction))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 6: direction - must be integer or None");
		return NULL;
	    }
	    l_direction = PyInt_AsLong(ar_direction);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_direction = &l_direction;
	}

	/* -------------------- */
	/* argument 7: [count] */
	if (ar_count == Py_None)
	{
	    al_count = 0;		/* omitted */
	    l_count  = 0;
	}
	else
	{
	    if (!PyInt_Check(ar_count))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 7: count - must be integer or None");
		return NULL;
	    }
	    l_count = PyInt_AsLong(ar_count);
	    if (PyErr_Occurred())
	    {
		return NULL;
	    }
	    al_count = &l_count;
	}

	/* -------------------- */
	l_status = smg$scroll_display_area
		(&l_display_id
		,al_start_row
		,al_start_column
		,al_height
		,al_width
		,al_direction
		,al_count
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    Py_INCREF(Py_None);
	    return (Py_None);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_smg_gr_error, 1, l_status);
} /* vms_smg_scroll_display_area () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SMG_SCROLL_DISPLAY_AREA.C */
