/* VMS_SYS_ASCUTC.C -- 16-JAN-2000 Uwe Zessin
   Python interface to SYS$ASCUTC()

12-DEC-1997 ZE. -- created in VMS_SYS.C
16-JAN-2000 ZE. -- extract into VMS_SYS_ASCUTC.C for V5.5-2 exclusion
*/

#ifdef __DECC
#pragma module VMS_SYS_ASCUTC "20000116"
#endif

#include "python.h"
#include "vmsdef.h"

#include <descrip.h>
#include <ssdef.h>		/* SS$_name	 */
#include <starlet.h>		/* SYS$name, ... */

/* ------------------------------------------------------------------------- */
extern PyObject *vms_sys_gr_error;	/* in VMS_SYS.C */
/* ------------------------------------------------------------------------- */

/* input a Python long integer, output a binary octaword */
extern long vms__cvt_pylong2octa (PyObject * r_longint, long * o_octaword);

/* ------------------------------------------------------------------------- */

char vms_sys_ascutc__doc[] =
"Convert UTC to ASCII\n\
timbuf = vms_sys.ascutc (utcadr [,cvtflg]).";

PyObject *
vms_sys_ascutc (PyObject *self, PyObject *args)
{
	PyObject	* ar_utcadr;
	long		  o_utcadr[4];		/* 128-bit utc_date_time */
	long		* ao_utcadr;

#ifndef S_TIMBUF
#define S_TIMBUF 23
#endif
	char			t_timbuf[S_TIMBUF];	/* ASCII time */
	struct dsc$descriptor_s r_timbuf;
	unsigned short int	w_timbuf_len;

	unsigned long		l_cvtflg;	/* conversion flag, $ASCUTC */

	unsigned long		l_status;

	/* -------------------- */
	ar_utcadr = Py_None;
	l_cvtflg  = 0;

	/* -------------------- */
	if (!PyArg_ParseTuple(args, "|Ol", &ar_utcadr, &l_cvtflg))
	{
	    return NULL;
	}

	/* -------------------- */
	/* argument 1: utcadr */
	if (ar_utcadr == Py_None)
	{
	    ao_utcadr = 0;
	}
	else
	{
	    if (!PyLong_Check(ar_utcadr))
	    {
		PyErr_SetString(PyExc_TypeError,
		    "argument 1: utcadr - must be long integer");
		return NULL;
	    }
	    /* convert Python long integer to binary octaword */
	    l_status = vms__cvt_pylong2octa (ar_utcadr, &o_utcadr[0]);
	    if (l_status != SS$_NORMAL)
	    {
		PyErr_SetString(PyExc_ValueError,
		   "argument 1: utcadr - conversion from long integer failed");
		return NULL;
	    }
	    ao_utcadr = &o_utcadr[0];
	}

	/* -------------------- */
	/* argument 2: cvtflg */
	/* already parsed or defaulted */

	/* -------------------- */
	/* set up descriptor */
	r_timbuf.dsc$w_length  = S_TIMBUF;
	r_timbuf.dsc$b_dtype   = DSC$K_DTYPE_T;
	r_timbuf.dsc$b_class   = DSC$K_CLASS_S;
	r_timbuf.dsc$a_pointer = &t_timbuf[0];

	/* -------------------- */
	l_status = sys$ascutc
		(&w_timbuf_len		/* [timlen] */
		,&r_timbuf		/* timbuf   */
		,ao_utcadr		/* [utcadr] */
		,l_cvtflg		/* [cvtflg] - by value */
		);

	/* -------------------- */
	if (l_status == SS$_NORMAL)
	{
	    /* return ASCII time to Python */
	    return Py_BuildValue
		("s#", &t_timbuf[0], (unsigned int)w_timbuf_len);
	}

	/* -------------------- */
	/* error */
	return PyVMS_ErrSetVal(vms_sys_gr_error, 1, l_status);
} /* vms_sys_ascutc () */

/* ------------------------------------------------------------------------- */

/* EOF: VMS_SYS_ASCUTC.C */
