/* VMSOBJ_XABSUM.C - 08-MAR-1999 ZE.
   SUMmary XAB (eXtended Access Block) object. 

   24-MAR-1999 ZE. -- replace some PyErr_SetString() with PyErr_Format()
   19-DEC-1999 ZE. -- allow "dir(xabsumobj)"
*/

#ifdef __DECC
#pragma module VMSOBJ_XABSUM "19991219"
#endif

/* ------------------------------------------------------------------------- */

#include "Python.h"
#include "vmsdef.h"

#include <xabsumdef.h>
#include "vmsobj__def.h"
#include "vmsobj__xabtst.h"	/* test for _any_ XAB */

/* ------------------------------------------------------------------------- */
/* Generic routine to convert binary data to a Python type. */

extern PyObject * vms__cvt_bin2py
        (char                    * ab_data_addr         /* data address */
        ,unsigned short int        w_data_type          /* data type	*/
        ,unsigned short int        w_bufsiz             /* data length	*/
        );
/* ------------------------------------------------------------------------- */
/* Generic routine to convert a Python type to binary data. */
extern long vms__cvt_py2bin
        (PyObject                * ar_py_data           /* Python data  */
        ,char                    * ab_data_addr         /* data address */
        ,unsigned short int        w_data_type          /* data type    */
        ,unsigned short int        w_bufsiz             /* data length  */
        );
/* ------------------------------------------------------------------------- */

PyTypeObject vmsobj_xabsum_type;

/* ------------------------------------------------------------------------- */
vmsobj_xabsum *
vmsobj_xabsum_new (struct XABSUM *arg)
{
	vmsobj_xabsum *xp;

	/* -------------------- */
	/* new 'vmsobj_xabsum' object */
	xp = PyObject_NEW (vmsobj_xabsum, &vmsobj_xabsum_type);
	if (xp == NULL)
	{
	    return NULL;
	}
	xp->l_flags = 0;
	/* pointers to other objects that the XABSUM points to */
	xp->ar_nxt    = NULL;	/* next XAB object */

	/* -------------------- */
	if (arg == NULL)
	{
	    /* allocate memory for real XABSUM and note it's address */
	    xp->ab_xabsum = malloc (XAB$K_SUMLEN);
	    if (xp->ab_xabsum == NULL)
	    {
		PyMem_DEL(xp);
		(void) PyErr_NoMemory();
		return NULL;
	    }
	    *xp->ab_xabsum = cc$rms_xabsum;
	    xp->l_flags |= VMSOBJ_M_ALLOC;
	}
	else
	{
	    /* 'arg' has address of already existing XABSUM */
	    xp->ab_xabsum = arg;
	    /* VMSOBJ_M_ALLOC is clear */
	}

	/* -------------------- */
	return xp;
}

/* ------------------------------------------------------------------------- */
/* vmsobj_xabsum methods */

static void
vo_xabsum_dealloc (vmsobj_xabsum *xp)
{
	/* was XABSUM allocated on object creation? */
	if (xp->l_flags & VMSOBJ_M_ALLOC)
	{
	    /* release memory of VMS XABSUM */
	    if (xp->ab_xabsum != NULL)
	    {
		(void) free (xp->ab_xabsum);
	    }
	}
	else
	{
	    /* The VMS XABSUM was not allocated by the object creation */
	    /* routine; its address was supplied to it.		       */
	    /* Remove pointers from XAB. */
	    xp->ab_xabsum->xab$l_nxt    = 0;
	}

	/* -------------------- */
	/* delete references to other objects the XABSUM might point to */
	if (xp->ar_nxt != NULL)
	{
	    Py_DECREF(xp->ar_nxt);	/* next XAB object */
	}

	/* -------------------- */
	/* release memory of object */
	PyMem_DEL(xp);
}

/* ------------------------------------------------------------------------- */
static PyMethodDef vo_xabsum_methods[] = {
	{NULL,		NULL}		/* sentinel */
};

/* ------------------------------------------------------------------------- */

struct vmsobj_xabsum_attr {
	char			* at_attrnam;	/* name of attribute */
	short int		  w_offset;	/* offset within XABSUM */
	unsigned short int	  w_typ;	/* data type	     */
	long			  l_mask;	/* bit address	     */
};
typedef struct vmsobj_xabsum_attr vmsobj_xabsum_attr;

/* XAB$B_COD + XAB$B_BLN are not changeable as they identify a XABSUM */

#define ROAT	0x8000	/* read-only attribute */
#define OBIO	0x4000	/* object I/O */
#define OFSMSK	0x1fff	/* mask byte offset count */

#define OB_K_NXT 1 /* next XAB object */

static vmsobj_xabsum_attr vmsobj_r_xabsum_attr [] = {
	{"B_BLN",	ROAT+1,	ITMTYP_K_BYTE,0	} ,
	{"B_COD",	ROAT+0,	ITMTYP_K_BYTE,0	} ,
	{"B_NOA",	8,	ITMTYP_K_BYTE,0	} ,
	{"B_NOK",	9,	ITMTYP_K_BYTE,0	} ,
	{"NXT",		OBIO+OB_K_NXT,
				ITMTYP_K_BOOL,0	} , /* object I/O */
	{"L_NXT",	ROAT+4,	ITMTYP_K_LONG,0	} ,

	{"W_PVN",	10,	ITMTYP_K_WORD,0	} ,
	{NULL,		0,	0,	0	}
};

/* ------------------------------------------------------------------------- */

static PyObject *
vo_xabsum_getattr (vmsobj_xabsum *xp, char *name)
{
	PyObject		  * ar_dict;	/* dictionary to be returned */
	PyObject		  * ar_dictobj;	/* temp. for insert into " */

	unsigned long l_attr_idx;

	/* -------------------- */
        if (strcmp(name, "__dict__") == 0)
	{
	    /* create a dictionary */
	    ar_dict = PyDict_New();
	    if (ar_dict == NULL)
	    {
		return NULL;
	    }

	    /* -------------------- */
	    l_attr_idx = 0;
	    while (vmsobj_r_xabsum_attr[l_attr_idx].at_attrnam != NULL)
	    {
		/* all values are currently "None" */
		ar_dictobj = Py_None;
		Py_INCREF(ar_dictobj);

		if (PyDict_SetItemString
		      (ar_dict
		      ,vmsobj_r_xabsum_attr[l_attr_idx].at_attrnam, ar_dictobj)
		    < 0)
		{
		    Py_DECREF(ar_dict);
		    Py_DECREF(ar_dictobj);
		    return NULL;
		}
		Py_DECREF(ar_dictobj);
		l_attr_idx++;		/* next attribute into __dict__ */
	    } /* loop over all attributes */

            return ar_dict;
        }

	/* -------------------- */
	l_attr_idx = 0;
	while (vmsobj_r_xabsum_attr[l_attr_idx].at_attrnam != NULL)
	{
	  if (strcmp (name, vmsobj_r_xabsum_attr[l_attr_idx].at_attrnam) == 0)
	  {
	    /* size of item types - in PYVMS_GLOBAL.C */
	    extern long vmsdef_gl_itmtypsiz[];
	    char * ab_data;

	    ab_data = (char *)xp->ab_xabsum;
	    /* ignore ROAT for get attribute */
	    /* if (vmsobj_r_xabsum_attr[l_attr_idx].w_offset >= 0) */

	    if (vmsobj_r_xabsum_attr[l_attr_idx].w_offset & OBIO)	    
	    {
	      /* object I/O */
	      unsigned short int  w_objtyp;

	      w_objtyp = vmsobj_r_xabsum_attr[l_attr_idx].w_offset & OFSMSK;
	      switch (w_objtyp)
	      {
	      /* ---------- */
	      case OB_K_NXT:
		/* return with a Python object, if one was assigned before */
		if (xp->ar_nxt != NULL)
		{
		    Py_INCREF (xp->ar_nxt);
		    return xp->ar_nxt;
		}
		Py_INCREF (Py_None);
		return Py_None;
	      /* ---------- */
	      default:
		return PyErr_Format(PyExc_SystemError,
			"BUGCHK - object I/O - unsupported object type: %d",
			(unsigned int)w_objtyp);
		return NULL;
	      }
	    } /* object I/O */
	    else
	    { /* BWL,M  I/O */
	      ab_data += (vmsobj_r_xabsum_attr[l_attr_idx].w_offset & OFSMSK);
	      if (vmsobj_r_xabsum_attr[l_attr_idx].l_mask == 0)
	      {
		  /* return resultant-value */
		  return vms__cvt_bin2py
		    (ab_data				    /* data address */
		    ,vmsobj_r_xabsum_attr[l_attr_idx].w_typ /* data type    */
		    ,vmsdef_gl_itmtypsiz[		    /* data length  */
			vmsobj_r_xabsum_attr[l_attr_idx].w_typ
					]
		    );
	      }
	      else
	      {
		/* return a BIT */
		if ((*ab_data) & vmsobj_r_xabsum_attr[l_attr_idx].l_mask)
		{
		  return PyInt_FromLong( (long)1);
		}
		else
		{
		  return PyInt_FromLong( (long)0);
		}
	      }
	    } /* object or BLW,M I/O */
	  } /* attribute check */
	  l_attr_idx++;
	} /* while() - attribute loop */

	/* -------------------- */
	/* fallback */
	return Py_FindMethod(vo_xabsum_methods, (PyObject *)xp, name);
} /* vo_xabsum_getattr (vmsobj_xabsum *xp, char *name) */

/* ------------------------------------------------------------------------- */

static int
vo_xabsum_setattr (vmsobj_xabsum *xp, char *name, PyObject *v)
{
	unsigned long l_attr_idx;

	/* -------------------- */
        if (strcmp(name, "__dict__") == 0)
	{
	    PyErr_SetString(PyExc_TypeError, "read-only special attribute");
	    return -1;
        }

	/* -------------------- */
	l_attr_idx = 0;
	while (vmsobj_r_xabsum_attr[l_attr_idx].at_attrnam != NULL)
	{
	  if (strcmp (name, vmsobj_r_xabsum_attr[l_attr_idx].at_attrnam) == 0)
	  {
	    /* size of item types - in PYVMS_GLOBAL.C */
	    extern long   vmsdef_gl_itmtypsiz[];
	    long	  l_status;
	    char	* ab_data;
	    PyObject	* ar_v;

	    ab_data = (char *)xp->ab_xabsum;

	    if ( !(vmsobj_r_xabsum_attr[l_attr_idx].w_offset & ROAT))
	    {
	      if (vmsobj_r_xabsum_attr[l_attr_idx].w_offset & OBIO)	    
	      {
		/* object I/O */
		unsigned short int   w_objtyp;

		w_objtyp = vmsobj_r_xabsum_attr[l_attr_idx].w_offset & OFSMSK;
	        switch (w_objtyp)
	        {
		/* ---------- */
		case OB_K_NXT:
		  if (v == Py_None)
		  {
		    /* 'None' = break connection to next XAB */
		    if (xp->ar_nxt != NULL)
		    {
		      Py_DECREF(xp->ar_nxt);  /* drop old reference */
		      xp->ar_nxt = NULL;
		    }
		    xp->ab_xabsum->xab$l_nxt = 0;
		    return 0; /* OK */
		  }
		  /* check and make sure this is _a_ vmsobj_xabXXX */
		  if (! vmsobj__anyxab_Check(v))
		  {
		    PyErr_SetString(PyExc_TypeError,
			"must be a vmsobj_xabXXX object or None");
		    return -1;
		  }
		  if (xp->ar_nxt != NULL)
		  {
		    Py_DECREF(xp->ar_nxt);  /* drop old reference */
		  }
		  xp->ar_nxt = v;		/* remember for Py_DECREF */
		  /* keep reference, because vmsobj_xabsum points to object */
		  Py_INCREF (v);
		  /* A different XAB than XABSUM might have been passed. */
		  /* All XAB objects must share the VMS' XAB address in  */
		  /* their structures. */
		  /* (char*) to keep DEC-C on Alpha quiet */
		  xp->ab_xabsum->xab$l_nxt =
					(char *)((vmsobj_xabsum *)v)->ab_xabsum;
		  return 0; /* OK */
		/* ---------- */
		default:
		  (void) PyErr_Format(PyExc_SystemError,
			"BUGCHK - object I/O - unsupported object type: %d",
			(unsigned int)w_objtyp);
		  return -1;
		}
	      } /* object I/O */
	      else
	      { /* BWL,M  I/O */
		ab_data += (vmsobj_r_xabsum_attr[l_attr_idx].w_offset & OFSMSK);
		if (vmsobj_r_xabsum_attr[l_attr_idx].l_mask == 0)
		{
/* @@ check that BW don't overflow */
                    /* convert Python type */
                    l_status = vms__cvt_py2bin
                    (v					    /* Python data  */
		    ,ab_data				    /* data address */
                    ,vmsobj_r_xabsum_attr[l_attr_idx].w_typ /* data type    */
                    ,vmsdef_gl_itmtypsiz[		    /* data length  */
		     vmsobj_r_xabsum_attr[l_attr_idx].w_typ
				      ]
		    );
		    /* .. */
		}
		else
		{
		  long			l_v;
		  unsigned long		l_msk;
		  unsigned short int	w_msk;
		  unsigned char		b_msk;

		  /* set/clear a BIT */
		  if (!PyInt_Check(v))
		  {
		    PyErr_SetString(PyExc_AttributeError,
			"bitmask assignment needs integer");
		    return -1;
		  }
		  l_v = PyInt_AsLong(v);
		  if (PyErr_Occurred())
		  {
		    return -1;
		  }

		  if (l_v == 0)
		  {
		    /* clear bit */
		    l_msk = ~ vmsobj_r_xabsum_attr[l_attr_idx].l_mask;
		    w_msk = l_msk;
		    b_msk = w_msk;

		    if (vmsobj_r_xabsum_attr[l_attr_idx].w_typ == ITMTYP_K_BYTE)
		    {
		      *ab_data &= b_msk;
		    }
		    else
		    {
		      if (vmsobj_r_xabsum_attr[l_attr_idx].w_typ == ITMTYP_K_WORD)
		      {
		        *ab_data &= w_msk;
		      }
		      else
		      {
			if (vmsobj_r_xabsum_attr[l_attr_idx].w_typ == ITMTYP_K_LONG)
			{
			  *ab_data &= l_msk;
			}
			else
			{
			  /* @@ bad table */
			} /* -ITMTYP_K_LONG */
		      } /* -ITMTYP_K_WORD */
		    } /* -ITMTYP_K_BYTE */
		  }
		  else
		  {
		    /* set bit */
		    l_msk = vmsobj_r_xabsum_attr[l_attr_idx].l_mask;
		    w_msk = l_msk;
		    b_msk = w_msk;

		    if (vmsobj_r_xabsum_attr[l_attr_idx].w_typ == ITMTYP_K_BYTE)
		    {
		      *ab_data |= b_msk;
		    }
		    else
		    {
		      if (vmsobj_r_xabsum_attr[l_attr_idx].w_typ == ITMTYP_K_WORD)
		      {
		        *ab_data |= w_msk;
		      }
		      else
		      {
			if (vmsobj_r_xabsum_attr[l_attr_idx].w_typ == ITMTYP_K_LONG)
			{
			  *ab_data |= l_msk;
			}
			else
			{
			  /* @@ bad table */
			} /* -ITMTYP_K_LONG */
		      } /* -ITMTYP_K_WORD */
		    } /* -ITMTYP_K_BYTE */
		  } /* clear or set bit */
	          l_status = 1; /* OK */
	        } /* others or bitmask */
	      } /* object or BWL,M I/O */
	    } /* offset 0x8000 indicates readonly attribute */
	    else
	    {
		PyErr_SetString(PyExc_AttributeError,
		    "read-only vmsobj_xabsum attribute");
		return -1;
	    }
	    if (l_status == 1)
	    {
	      l_status = 0;
	    }
	    return l_status;       /* possible error mesage has been set */
	  }
	  l_attr_idx++;
	}

	/* -------------------- */
	PyErr_SetString(PyExc_AttributeError,
	    "non-existing vmsobj_xabsum attribute");
	return -1;
} /* static int vo_xabsum_setattr (vmsobj_xabsum *xp, char *name, PyObject *v) */

/* ------------------------------------------------------------------------- */

static PyObject *
vo_xabsum_repr (vmsobj_xabsum *ar_xabsum)
{
	char buf[40];

	sprintf(buf, "<vmsobj_xabsum, XABSUM at 0x%08x>", (ar_xabsum->ab_xabsum));
	return PyString_FromString(buf);
} /* static PyObject * vo_xabsum_repr (vmsobj_xabsum *ar_xabsum) */

/* ------------------------------------------------------------------------- */
PyTypeObject vmsobj_xabsum_type = {
	PyObject_HEAD_INIT(&PyType_Type)
	0,				/*ob_size*/
	"vmsobj_xabsum",		/*tp_name*/
	sizeof(vmsobj_xabsum),		/*tp_basicsize*/
	0,				/*tp_itemsize*/
	/* methods */
	(destructor)vo_xabsum_dealloc,	/*tp_dealloc*/
	(printfunc)0,			/*tp_print*/
	(getattrfunc)vo_xabsum_getattr,	/*tp_getattr*/
	(setattrfunc)vo_xabsum_setattr,	/*tp_setattr*/
	(cmpfunc)0,			/*tp_compare*/
	(reprfunc)vo_xabsum_repr,	/*tp_repr*/
	0,				/*tp_as_number*/
	0,				/*tp_as_sequence*/
	0,				/*tp_as_mapping*/
	(hashfunc)0,			/*tp_hash*/
};

/* ------------------------------------------------------------------------- */

/* EOF: VMSOBJ_XABSUM.C */
